// Copyright (c) 2014–2015 Mattt Thompson (http://mattt.me)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

import Foundation
import Upsurge
import XCTest

class AuxiliaryTests: XCTestCase {
    let n = 10000

    func testCopysign() {
        let signs = RealArray((0..<n).map {$0 % 2 == 0 ? 1.0 : -1.0})

        let magnitudes = RealArray(count: n)
        for i in 0..<n {
            magnitudes[i] = Real(arc4random_uniform(10))
        }

        let expected = RealArray(count: n)
        for (i, (sign, magnitude)) in Zip2Sequence(signs, magnitudes).enumerate() {
            expected[i] = sign * abs(magnitude)
        }

        var actual: RealArray = []
        self.measureBlock {
            actual = copysign(signs, magnitude: magnitudes)
        }

        XCTAssert(actual == expected)
    }
}
